@@ -37,7 +37,6 @@ import com.google.protobuf.Descriptors.FieldDescriptor;
 import com.google.protobuf.DynamicMessage;
 import com.google.protobuf.Extension;
 import com.google.protobuf.Message;
-
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
@@ -76,7 +75,7 @@ public class ProtoTypeAdapter
   /**
    * Determines how enum <u>values</u> should be serialized.
    */
-  public static enum EnumSerialization {
+  public enum EnumSerialization {
     /**
      * Serializes and deserializes enum values using their <b>number</b>. When this is used, custom
      * value names set on enums are ignored.
@@ -117,12 +116,12 @@ public class ProtoTypeAdapter
      * For example, if you use the following parameters: {@link CaseFormat#LOWER_UNDERSCORE},
      * {@link CaseFormat#LOWER_CAMEL}, the following conversion will occur:
      *
-     * <pre>
+     * <pre>{@code
      * PROTO     <->  JSON
      * my_field       myField
      * foo            foo
      * n__id_ct       nIdCt
-     * </pre>
+     * }</pre>
      */
     public Builder setFieldNameSerializationFormat(CaseFormat fromFieldNameFormat,
         CaseFormat toFieldNameFormat) {
@@ -192,7 +191,7 @@ public class ProtoTypeAdapter
   private static final com.google.protobuf.Descriptors.FieldDescriptor.Type ENUM_TYPE =
       com.google.protobuf.Descriptors.FieldDescriptor.Type.ENUM;
 
-  private static final ConcurrentMap<String, Map<Class<?>, Method>> mapOfMapOfMethods =
+  private static final ConcurrentMap<String, ConcurrentMap<Class<?>, Method>> mapOfMapOfMethods =
       new MapMaker().makeMap();
 
   private final EnumSerialization enumSerialization;
@@ -309,7 +308,7 @@ public class ProtoTypeAdapter
             }
           }
         }
-        return (Message) protoBuilder.build();
+        return protoBuilder.build();
       } catch (SecurityException e) {
         throw new JsonParseException(e);
       } catch (NoSuchMethodException e) {
@@ -389,7 +388,7 @@ public class ProtoTypeAdapter
       EnumValueDescriptor fieldValue = desc.findValueByNumber(jsonElement.getAsInt());
       if (fieldValue == null) {
         throw new IllegalArgumentException(
-            String.format("Unrecognized enum value: %s", jsonElement.getAsInt()));
+            String.format("Unrecognized enum value: %d", jsonElement.getAsInt()));
       }
       return fieldValue;
     }
@@ -397,10 +396,10 @@ public class ProtoTypeAdapter
 
   private static Method getCachedMethod(Class<?> clazz, String methodName,
       Class<?>... methodParamTypes) throws NoSuchMethodException {
-    Map<Class<?>, Method> mapOfMethods = mapOfMapOfMethods.get(methodName);
+    ConcurrentMap<Class<?>, Method> mapOfMethods = mapOfMapOfMethods.get(methodName);
     if (mapOfMethods == null) {
       mapOfMethods = new MapMaker().makeMap();
-      Map<Class<?>, Method> previous =
+      ConcurrentMap<Class<?>, Method> previous =
           mapOfMapOfMethods.putIfAbsent(methodName, mapOfMethods);
       mapOfMethods = previous == null ? mapOfMethods : previous;
     }
